<?php
namespace Teknisa\MAN\Controllers;

use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DTO;
use Teknisa\MAN\Services\ConfigMesaAmbienteService;
use Zeedhi\Framework\DTO\Response\Notification;

class ConfigMesaController extends CrudWithParameterBag  {

    const MESA_AUTOCAD_DATASOURCE = 'vnd56000_configmesaambiente_mesa_autocad';
    const MESA_DATASOURCE = 'vnd56000_configmesaambiente_mesa';
    
    const MESA_COLUMNS = array();
    
    protected $dataSourceName = ConfigMesaController::MESA_DATASOURCE;
    protected $parameterBagColumns = ConfigMesaController::MESA_COLUMNS;
    
    protected $ConfigMesaAmbienteService;
    
    public function __construct(Manager $dataSourceManager, ParameterBag $parameterBag, ConfigMesaAmbienteService $ConfigMesaAmbienteService ) {
        parent::__construct($dataSourceManager, $parameterBag);
        $this->ConfigMesaAmbienteService = $ConfigMesaAmbienteService;
    }   
    
    public function saveMesa(DTO\Request\DataSet $request, DTO\Response $response) {
        $rows = $request->getDataSet()->getRows();
        if(count($rows) > 1) {
            $rows[0] = end($rows);
            $tam = count($rows);
            for($i = 1; $i < $tam; $i++) {
                array_pop($rows);
            }
        }
        
        $row = $rows[0];
        if($row['__is_new']) {
            $verify = $this->ConfigMesaAmbienteService->getMesa (
                $row['NRORG'],
                $row['CDFILIAL'],
                $row['CDLOJA'],
                $row['NRMESA']
            );
            if(empty($verify)){
                $this->saveWithoutNotification($request, $response);
            }else {
                throw new \Exception('Esta mesa já está cadastrada na sala ' . $verify[0]['CDSALA']); 
            }
        }else{
            $this->saveWithoutNotification($request, $response);
        }
    }
    
    // To save this method use just one row and changes it NRMESA, according to 
    // NRMESAINI and NRMESAFIN. 
    // This method also jumps registries if there's something between NRMESAINI and NRMESAFIN...
    
    public function saveMesas(DTO\Request\DataSet $request, DTO\Response $response) {
        $this->dataSourceName = ConfigMesaController::MESA_AUTOCAD_DATASOURCE;
        
        $rows = $request->getDataSet()->getRows();
        if(count($rows) > 1) {
            throw new \Exception("Não é possível inserir mais de um auto cadastro de uma vez");
        }
        
        $row = $rows[0];
        $rows = [];
        
        try {
            
            // Sets the IDATIVO of "all rows"
            $row['IDATIVO'] = 'S';
            
            // Array with the 'MESAS' to be inserted
            $mesas = array();
            $i = 0;
            while( intval($row['NRMESAINI']) + $i <= intval($row['NRMESAFIN'])) {
                $mesas[] = intval($row['NRMESAINI'])  + $i;
                $i++;
            }
            
            $intervalRows = $this->ConfigMesaAmbienteService->getMesasInterval (
                $row['NRORG'],
                $row['CDFILIAL'],
                $row['CDLOJA'],
                $row['CDSALA'],
                $row['NRMESAINI'],
                $row['NRMESAFIN']
            );
            
            $rowsSaved  = array();
            foreach ($intervalRows as $intervalRow) {
                $rowsSaved[] = intval($intervalRow['NRMESA']); 
            }
                
            $aux = array_diff($mesas, $rowsSaved);
            
            if(count($rowsSaved) == 0 ) {
                
                foreach($aux as $key => $value) {
                    $row['NRMESA'] = sprintf("%04d", $value );
                    $row['NMMESA'] = 'Mesa '.$value; 
                    array_push($rows, clone $row);
                }
                
                $newRequest = new DTO\Request\DataSet(
                    new DataSet(
                        $this->dataSourceName,
                        $rows
                    ),
                    $request->getMethod(),
                    $request->getRoutePath(),
                    $request->getUserId()
                );
     
                $this->saveWithoutNotification($newRequest, $response);
                $response->addDataSet(new DataSet($this->dataSourceName, $rows));
            } else {
                throw new \Exception('A mesa '.$rowsSaved[0].' já está cadastrada!');    
            }
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        } finally {
            $this->dataSourceName = ConfigMesaController::MESA_DATASOURCE;
        }
    }
    
    public function getMesas(DTO\Request\Filter $request, DTO\Response $response) {
        $this->dataSourceName = $this::MESA_DATASOURCE;
        $this->parameterBagColumns = array('NRORG', 'CDSALA', 'CDLOJA', 'CDFILIAL');
        parent::find($request, $response);
    }
    
    private function saveWithoutNotification($request, $response) {
	   	try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $persistedRows = $this->dataSourceManager->persist($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $persistedRows));
            $nrPersistedRows = count($persistedRows);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
	}
	
	public function deleteWithoutNotification ($request, $response) {
		try {
            $dataSet = $request->getDataSet();
            $dataSet->setDataSourceName($this->dataSourceName);
            $deletedRows = $this->dataSourceManager->delete($dataSet);
            $response->addDataSet(new DataSet($this->dataSourceName, $deletedRows));
            $nrDeletedRows = count($deletedRows);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
	}
    
     public function getNrMesa(DTO\Request\Row $request, DTO\Response $response) {
        $row = $request->getRow();
        
        $data = $this->ConfigMesaAmbienteService->getNrMesa ($row['NRORG'],$row['CDFILIAL'],$row['CDLOJA']);
        $response->addDataSet(new DataSet("MaxNRMESA", $data));
     }
    
}