<?php
namespace Teknisa\MAN\Controllers;


use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Message;
use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DTO\Response\Error;
use Teknisa\MAN\Services;
use Teknisa\MAN\Utilities;
use \Datetime;

class ParamAutomatizador  {

	
	public function __construct ($entityManager, $environment, Services\ParamAutomatizadorService $automatizadorService, $utilService) {
        $this->entityManager    = $entityManager;
        $this->environment      = $environment;
        $this->automatizadorService    = $automatizadorService;
        $this->utilService    = $utilService;
    }
    
    public function find(DTO\Request\Filter $request, DTO\Response $response) {
        $arrayretorno = array();
        $arrayretorno = array(
            [
                "CDROTINA" => 'vnd1400',
                'NMROTINA' => 'Consolidação de vendas'
            ],
            [
                "CDROTINA" => 'vnd06200',
                'NMROTINA' => 'Previsão de estoque'
            ]
        );
            
        $response->addDataSet(new DataSet('paraAutomatizador',$arrayretorno));
    }

    public function findUnidade(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $this->utilService->getParamsAsArray($request);
        $unidades = $this->automatizadorService->findUndiade($conditions);
        $response->addDataSet(new DataSet('paraAutomatizador',$unidades));
    }
    
    public function findPeriodoCron(DTO\Request\Filter $request, DTO\Response $response){
        $conditions = $this->utilService->getParamsAsArray($request);
        
        $arrayData =  array(
                                    [
                                        "CDCRON"=> "*/10 * * * *", 
                                        "DSCRON"=> "A cada 10 minutos"
                                    ],
                                    [
                    			        "CDCRON"=> "*/30 * * * *", 
                    			        "DSCRON"=> "Duas vezes por Hora"
                                    ],
                    			    [
                    			        "CDCRON"=> "0 * * * *",    
                    			        "DSCRON"=> "Uma vez por Hora"
                    			    ],
                    			    [
                    			        "CDCRON"=> "0 0,12 * * *", 
                    			        "DSCRON"=> "Duas vezes por dia"
                    			    ],
                    			    [
                    			        "CDCRON"=> "0 0 * * 0",
                    			        "DSCRON"=> "Semanal"
                    			    ],
                    			    [
                    			        "CDCRON"=> "0 0 1,15 * *", 
                    			        "DSCRON"=> "15 em 15 dias"
                    			    ],
                    			    [
                    			        "CDCRON"=> "0 0 1 * *",    
                    			        "DSCRON"=> "Uma vez por Mês"
                    			    ],
                    			    [
                    			        "CDCRON"=> "0 0 1 1 *",    
                    			        "DSCRON"=> "Uma vez por Ano"
                    			    ]
                                );
        
        $response->addDataSet(new DataSet('paraAutomatizador_cron',$arrayData));
    }
    
    public function saveUnidade(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            $rows = $request->getDataSet()->getRows()[0];
            return $this->automatizadorService->saveUnidade($rows);
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
	}
	
	public function saveUnidadeCadAuto(DTO\Request\DataSet $request, DTO\Response $response){
    	try{
            $rows = $request->getDataSet()->getRows()[0];
            $this->automatizadorService->deleteUnidadeCadAuto($rows);
            return $this->automatizadorService->saveUnidadeCadAuto($rows);
    	} catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
	}
	
	public function deleteUnidade(DTO\Request\DataSet $request, DTO\Response $response){
    	try{
            $rows = $request->getDataSet()->getRows()[0];
            return $this->automatizadorService->deleteUnidade($rows, $rows['CDFILIAL']);
    	} catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
	}
	
	public function removeUnidadeCadAuto(DTO\Request\DataSet $request, DTO\Response $response){
    	try{
            $rows = $request->getDataSet()->getRows();
            return $this->automatizadorService->deleteUnidade($rows);
    	} catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode(), $e->getTraceAsString()));
        }
	}

}