<?php
namespace Teknisa\MAN\Controllers;


use Zeedhi\Framework\Controller\CrudWithParameterBag;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Response\Notification;
use Teknisa\MAN\Services\ObservacaoDesconto;
use Teknisa\MAN\Services;
use Teknisa\MAN\Utilities;

class RelObservacaoDesconto extends CrudWithParameterBag{
    
    const OBS_DESCONTO = 'vnd0160K_relObservacaoDesconto';
    
    protected $dataSourceManager;
    protected $parameterBag;
    protected $observacaoDesconto;
    protected $utilService;
    
    protected $parameterBagColumns = array('CDFILIAL', 'CDLOJA', 'TCDFILIAL', 'TCDLOJA', 'DTVENDAINI', 'DTVENDAFIN');
    
    public function __construct ( Manager $dataSourceManager, ParameterBag $parameterBag, ObservacaoDesconto $observacaoDesconto, Utilities\Util $utilService ){
        parent::__construct($dataSourceManager, $parameterBag);
        $this->utilService = $utilService;
        $this->observacaoDesconto = $observacaoDesconto;
    }
    
    public function getData(DTO\Request\Filter $request, DTO\Response $response) {
  
        ini_set('memory_limit', '1G');
        set_time_limit(120);
        try{
            $conditions = $this->utilService->getParamsAsArray($request);
            $this->dataSourceName = self::OBS_DESCONTO;
            
            $conditions['CDFILIAL'] = implode($conditions['CDFILIAL'], '_');
            $conditions['TCDFILIAL'] = null;
            if( array_key_exists('NMFILIAL', $conditions) ) {
                unset($conditions['NMFILIAL']);
            }
            
            if( array_key_exists('DTENTRVENDA', $conditions) ) {
                $conditions['DTVENDAINI'] = $conditions['DTENTRVENDA'][0];
                $conditions['DTVENDAFIN'] = $conditions['DTENTRVENDA'][1];
                
                unset($conditions['DTENTRVENDA']);
            }
            
            if(!empty($conditions['CDLOJA'])) {
                $conditions['CDLOJA'] = implode($conditions['CDLOJA'], '_');
                $conditions['TCDLOJA'] = null;
            } else {
                $conditions['CDLOJA'] = null;
                $conditions['TCDLOJA'] = 'T';
            }
            unset($conditions['NRORG']);
            $dataSourceFilter = array();
            foreach($conditions as $key => $value) {
                $dataSourceFilter[] = $condition = array(
                    "columnName" => $key,
                    "operator"=> "=",
                    "value"=> $value
                );
            }
            $newRequest = new DTO\Request\Filter(
                new FilterCriteria(
                    $request->getFilterCriteria()->getDataSourceName(),
                    $dataSourceFilter,
                    $request->getFilterCriteria()->getPage(),
                    $request->getFilterCriteria()->getPageSize()
                ),
                $request->getMethod(),
                $request->getRoutePath(),
                $request->getUserId()
            );
            
            parent::find($newRequest, $response);
            
            /* Older Code
            if($conditions['CDFILIAL']){
                if(!array_key_exists('CDFILIAL', $conditions)) {
                    $conditions['CDFILIAL'] = array();
                }
                if(!array_key_exists('CDLOJA', $conditions)) {
                    $conditions['CDLOJA'] = array();
                }
                if(array_key_exists('DTENTRVENDA', $conditions)) {
                    $conditions['P_DTINI'] = $conditions['DTENTRVENDA'][0];
                    $conditions['P_DTFIM'] = $conditions['DTENTRVENDA'][1];
                    unset($conditions['DTENTRVENDA']);
                }
                $response->addDataSet(new DataSet("vnd0160K_relObservacaoDesconto", 
                $this->observacaoDesconto->buscaData($conditions['CDFILIAL'], $conditions['P_DTINI'], $conditions["P_DTFIM"], $conditions["CDLOJA"], $conditions["NRORG"])));
                
                return $response;
            }*/
            
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
}